/**
 * Name:    JimController.java
 * Created: Dec 7, 2010
 * 
 * @author: relation
 */
package sk.fiit.testframework.communication;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetSocketAddress;

import org.apache.commons.net.tftp.TFTPClient;

import sk.fiit.testframework.annotations.Reviewed;
import sk.fiit.testframework.annotations.TestCovered;

/**
 * Used for remote controlling of agent Jim through TFTP protocol.
 * 
 * 
 * @author relation
 *
 */
public class JimController {

	private TFTPClient tftpClient = new TFTPClient();

	/**
	 * Sends a file to remote TFTP server. 
	 * 
	 * @param host	Remote TFTP server.
	 * @param saveFileAs	Determines how file on remote TFTP server will be 
	 * 						saved.
	 * @param inputStream	Input stream of data.
	 * @throws IOException	
	 */
	@TestCovered
	public void sendFile(InetSocketAddress host, String saveFileAs, InputStream inputStream) throws IOException {
		tftpClient.open();
		tftpClient.sendFile(saveFileAs, TFTPClient.BINARY_MODE, inputStream, host.getAddress(), 
				host.getPort());
		tftpClient.close();
	}
	
	/**
	 * Invokes executing of ruby script on special TFTP server.
	 * Used for internal purposes with agent Jim.
	 * 
	 * @param host	Remote TFTP server.
	 * @param script	UTF-8 String text to be executed on server side.
	 * @throws IOException
	 */
	@TestCovered
	public void executeRubyScriptOnAgent(InetSocketAddress host, String script) throws IOException {
		ByteArrayInputStream bais = new ByteArrayInputStream(
				script.getBytes("UTF-8"));
		
		tftpClient.open();
		tftpClient.sendFile("ruby.exec", TFTPClient.BINARY_MODE, bais, host.getAddress(), 
				host.getPort());
		tftpClient.close();
	}
	
}
