package sk.fiit.testframework.monitor;

import java.io.*;
import java.net.*;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;

import sk.fiit.testframework.annotations.*;
import sk.fiit.testframework.parsing.MessageParserFactory;
import sk.fiit.testframework.parsing.models.messages.Message;
import sk.fiit.testframework.parsing.models.messages.MessageType;
import sk.fiit.testframework.worldrepresentation.models.*;
import sk.fiit.testframework.worldrepresentation.*;

public class Monitor {

    byte[] buffer = new byte[100000];
    int maxToRead = 1024;
    public Socket socket;
    String serverIp;
    Integer port;
    private DataInputStream input;
    private SimulationState state;
    private MessageInterpreter messageInterpreter = new MessageInterpreter();

    public Monitor(String serverIp, int port) {
        this.serverIp = serverIp;
        this.port = port;
        this.state = new SimulationState();
        try {
            socket = new Socket(serverIp, port);
        } catch (UnknownHostException ex) {
            Logger.getLogger(Monitor.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(Monitor.class.getName()).log(Level.SEVERE, null, ex);
        }

    }

    public void start() throws IOException {
        try {

            input = new DataInputStream(socket.getInputStream());
        } catch (Exception e) {
            System.exit(-1);
        }

        mainLoop();
    }

    @UnderConstruction
    private void mainLoop() throws IOException {
        while (true) {
//                    System.out.println("while (true)");
            while (input.available() == -1) {
                Thread.yield();
            }
//                        System.out.println("before recieve()");
            String incoming = receive();

//			System.out.println(incoming);
            if (incoming.length() > 0) {
                Message message = MessageParserFactory.getParser(incoming).parse(incoming);
//                                System.out.println("interpret start");
                messageInterpreter.interpret(message, state);
//                                System.out.println("interpret stop");

                if (message.getType() == MessageType.GameState) {
                    setInformationComplete(true);
                }
            }

        }
    }

    private String receive() throws IOException {
        // first number of the message denotes the length of the message
//		System.out.println("input.readInt();");
        int messageLength = input.readInt();
//                System.out.println("after input.readInt();");

        if (Math.abs(messageLength) > buffer.length) {
            return "";
        }

        int bytesAlreadyRead = 0;
        int toRead = maxToRead > messageLength ? messageLength : maxToRead;
        while (messageLength > bytesAlreadyRead) {
            bytesAlreadyRead += input.read(buffer, bytesAlreadyRead, toRead);
            toRead = maxToRead > messageLength - bytesAlreadyRead ? messageLength - bytesAlreadyRead : maxToRead;
        }

        String incoming = new String(buffer, 0, messageLength);
        return incoming;
    }

    public void stop() {
    }
    private boolean informationComplete;

    /**
     * @return the informationComplete
     */
    public boolean isInformationComplete() {
        return informationComplete;
    }

    /**
     * @param informationComplete the informationComplete to set
     */
    public void setInformationComplete(boolean informationComplete) {
        this.informationComplete = informationComplete;
    }

    public SimulationState getSimulationState() {
        return state;
    }
}
