/**
 * Name:    Trainer.java
 * Created: Feb 27, 2011
 * 
 * @author: relation
 */
package sk.fiit.testframework.trainer;

import java.io.IOException;
import java.net.Socket;
import sk.fiit.testframework.annotations.UnderConstruction;

import sk.fiit.testframework.communication.JimController;
import sk.fiit.testframework.geometry.Point3D;
import sk.fiit.testframework.parsing.models.PlayMode;
import sk.fiit.testframework.trainer.testsuite.AgentJim;

/**
 * <code>Trainer</code> is used for sending commands to RoboCup 3D simulation
 * soccer server with aim to set specific state on server.
 * 
 * @author relation
 */
@UnderConstruction
public class Trainer {

	private RobocupServerAddress address;
	private Socket socket;

	public Trainer(RobocupServerAddress address) throws IOException {
		this.address = address;
		this.socket = new Socket(address.getServerSocketAddress().getAddress(),
				address.getServerMonitorAddress().getPort());

	}

	public Trainer(RobocupServerAddress address, Socket s) {
		this.address = address;
		this.socket = s;
	}

	private void performCommand(TrainerCommand command) throws IOException {
		if (!socket.isConnected()) {
			socket.connect(address.getServerMonitorAddress());
		}

		socket.getOutputStream().write(command.getAsBytes());
		socket.getOutputStream().flush();
	}

	public void setAgentPosition(AgentJim agent, Point3D position)
			throws IOException {
		TrainerCommand tc = new TrainerCommand.Agent(position, new Point3D(0,
				0, 0), agent.getAgentData().getTeam().toString(), agent
				.getAgentData().getUniformNumber());
		System.out.println(tc.getCommand());
		performCommand(tc);
	}

	public void setAgentPosition(String team, int playerid, Point3D position)
			throws IOException {
		TrainerCommand tc = new TrainerCommand.Agent(position, new Point3D(0,
				0, 0), team, playerid);
		System.out.println(tc.getCommand());
		performCommand(tc);
	}

	public void setBallPosition(Point3D position) throws IOException {
		TrainerCommand tc = new TrainerCommand.BallPosition(position);
		performCommand(tc);
	}

	public void setBallVelocity(Point3D velocity) throws IOException {
		TrainerCommand tc = new TrainerCommand.BallVelocity(velocity);
		performCommand(tc);
	}

	public void setBall(Point3D position, Point3D velocity) throws IOException {
		TrainerCommand tc = new TrainerCommand.Ball(position, velocity);
		performCommand(tc);
	}

	public void setPlayMode(PlayMode mode) throws IOException {
		TrainerCommand tc = new TrainerCommand.PlayMode(mode);
		performCommand(tc);
	}

	/**
	 * Invokes replan action on remote agent through TFTP protocol.
	 * 
	 * @param agent
	 */
	public void invokeReplan(AgentJim agent) throws IOException {
		StringBuilder sb = new StringBuilder();
		sb.append("require \"java\"\n");
		sb.append("include_class \"sk.fiit.jim.init.ScriptBoot\"\n");
		sb.append("ScriptBoot.boot");
		executeRubyScript(agent, sb.toString());
	}

	public void invokeRestart(AgentJim agent) throws IOException {
		StringBuilder sb = new StringBuilder();
		sb.append("require \"java\"\n");
		sb.append("include_class \"sk.fiit.jim.agent.communication.CommunicationThread\"\n");
		sb.append("CommunicationThread.request_restart");
		executeRubyScript(agent, sb.toString());
	}

	private JimController controller = new JimController();

	public void executeRubyScript(AgentJim agent, String script)
			throws IOException {
		controller.executeRubyScriptOnAgent(agent.getTFTPAddress(), script);
	}

}
