/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sk.fiit.testframework.trainer.testsuite;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.InetSocketAddress;
import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.ListIterator;
import sk.fiit.testframework.annotations.UnderConstruction;
import sk.fiit.testframework.trainer.RobocupServerAddress;
import sk.fiit.testframework.trainer.Team;

/**
 *
 * @author relation
 */
@UnderConstruction
public class AgentJim implements Runnable {

    public static class Data {
        private int uniformNumber;
        private Team team;
        private int tftpPort;

        public Data(int uniformNumber, Team team, int tftpPort) {
            this.uniformNumber = uniformNumber;
            this.team = team;
            this.tftpPort = tftpPort;
        }

        public Team getTeam() {
            return team;
        }

        public int getTftpPort() {
            return tftpPort;
        }

        public int getUniformNumber() {
            return uniformNumber;
        }

    }

    private RobocupServerAddress serverAddress;
    private File agentRunnableJarPath;
    private Data agentData;
    
    private List<String> processCommand = new ArrayList<String>();
    private File workingDirectory;

    public Data getAgentData() {
        return agentData;
    }

    /**
     * Constructor used when running Jim agent from sources.
     * 
     * @param agentRunnableJarPath
     * @param serverAddress
     * @param agentData
     * @throws IOException
     */
//    public AgentJim(File agentRunnableJarPath, RobocupServerAddress serverAddress, Data agentData) throws IOException {
//        this.serverAddress = serverAddress;
//        this.agentRunnableJarPath = agentRunnableJarPath;
//        this.agentData = agentData;
//    }

    private List<String> createProcessCommand(File jimBinDirRoot, Data agentData) {

        final String javaCommand = "java";
        final String encodingParam = "-Dfile.encoding=UTF-8";
        final String classPathParam = "-classpath";
        final String mainClass = "sk.fiit.jim.init.TestFrameworkMain";

        List<String> command = new ArrayList<String>();
        command.add(javaCommand);
        command.add(encodingParam);

        File libDirectory = new File(jimBinDirRoot.getParentFile(), "lib");
        System.out.println(libDirectory.getAbsolutePath());

        List<File> libraries = getDirectoryFiles(libDirectory, ".jar");

        
        ListIterator<File> libraryIterator = libraries.listIterator();
        String classPath = new String();
        while (libraryIterator.hasNext()) {
            File library = libraryIterator.next();
            classPath += library.getAbsolutePath();
            if (libraryIterator.hasNext()) {
                classPath += ":";
            }
        }

        command.add(classPathParam);
        command.add(classPath);

        command.add(mainClass);
        // Agent specific running
        command.add(String.valueOf(agentData.getTftpPort()));
        command.add(String.valueOf(agentData.getUniformNumber()));

        return command;
    }

    private List<File> getDirectoryFiles(File libPath, final String extension) {
        return Arrays.asList(libPath.listFiles(new FileFilter() {

            @Override
            public boolean accept(File pathname) {
                if (pathname.isFile() && pathname.getPath().toLowerCase().endsWith(extension)) {
                    return true;
                }
                else {
                    return false;
                }
            }
        }));
    }


    public AgentJim(File jimBinDirRoot, RobocupServerAddress serverAddress, Data agentData) {
        this.serverAddress = serverAddress;
        this.agentData = agentData;
        processCommand = createProcessCommand(jimBinDirRoot, agentData);
        workingDirectory = jimBinDirRoot;
    }

    /**
     * Constructor used when we want to attach to an running Jim agent 
     * connected to the server on address <code>serverAddress</code>. 
     * This is especially useful when debugging, because we don't have to 
     * run new instance of agent everytime we want to use testcase. We can
     * simply run agent in shell and by this constructor treat is like a
     * regular runned agent.
     * 
     * @param serverAddress
     * @param agentData
     * @throws IOException
     */

    public AgentJim(RobocupServerAddress serverAddress, Data agentData) throws IOException {
        this.serverAddress = serverAddress;
        this.agentData = agentData;
    }

    /*
     * Private methods
     */

    /*
     * Sure methods
     */
    public InetSocketAddress getTFTPAddress() {
        // TODO: Only localhost now
        return new InetSocketAddress("localhost", agentData.getTftpPort());
    }

    public boolean isConnectedToServer() {
        throw new UnsupportedOperationException();
    }

    public void connectToServer() {
        throw new UnsupportedOperationException();
    }

    public void disconnectFromServer() {
        throw new UnsupportedOperationException();
    }

    @Override
    public void run() {

        if (processCommand.isEmpty()) {
            // We are using constructor without need to run agent.
            return;
        }

        try {
            ProcessBuilder pb = new ProcessBuilder();
            pb.redirectErrorStream(true);
            pb.directory(workingDirectory);
            pb.command(processCommand);
            Process p = pb.start();

            BufferedReader br = new BufferedReader(new InputStreamReader(p.getInputStream()));
            String line = null;
            while ((line = br.readLine()) != null) {
                // TODO: Output system
                System.out.println(br.readLine());
            }
            p.waitFor();
        } catch (Exception ex) {
            // TODO: Output system
            System.out.println(ex);
        }
    }


//    @Override
//    public void run() {
//        System.out.println("Trying to run");
//
//        final boolean DEBUG_NON_JAR = true; // TODO: Working only on my computer!
//
//        String command[] = null;
//        File workingDir = null;
//
//        if (DEBUG_NON_JAR) {
//            command = new String[]{"/usr/lib/jvm/java-6-openjdk/bin/java", "-Dfile.encoding=UTF-8", "-classpath", "/home/relation/workspace/Jim Team05/bin:/home/relation/workspace/Jim Team05/lib/bsf.jar://home/relation/workspace/Jim Team05/lib/jruby-complete-1.4.0.jar:/home/relation/workspace/Jim Team05/lib/commons-logging-1.1.jar:/home/relation/workspace/Jim Team05/lib/junit.jar:/home/relation/workspace/Jim Team05/lib/hamcrest-all-1.3.0RC1.jar:/home/relation/workspace/Jim Team05/lib/mockito-all-1.8.5.jar:/home/relation/workspace/Jim Team05/lib/commons-net-2.2.jar:/home/relation/Apps/eclipse/plugins/org.aspectj.runtime_1.6.10.20101003133000/aspectjrt.jar",
//            "sk.fiit.jim.init.TestFrameworkMain", String.valueOf(agentData.getTftpPort()), String.valueOf(agentData.getUniformNumber())};
//            workingDir = new File("/home/relation/workspace/Jim Team05");
//        } else {
//            command = new String[]{"java", "-jar", agentRunnableJarPath.getAbsolutePath()};
//            workingDir = agentRunnableJarPath.getParentFile();
//        }
//        try {
//            ProcessBuilder pb = new ProcessBuilder();
//            pb.redirectErrorStream(true);
//            pb.directory(workingDir);
//            pb.command(command);
//            Process p = pb.start();
//
//            BufferedReader br = new BufferedReader(new InputStreamReader(p.getInputStream()));
//            String line = null;
//            while ((line = br.readLine()) != null) {
//                System.out.println(br.readLine());
//            }
//            p.waitFor();
//        } catch (Exception ex) {
//            System.out.println(ex);
//        }
//
//    }
}
