/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package sk.fiit.testframework.trainer.testsuite;

import java.io.IOException;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.logging.Level;
import java.util.logging.Logger;
import sk.fiit.testframework.annotations.UnderConstruction;
import sk.fiit.testframework.monitor.Monitor;
import sk.fiit.testframework.trainer.RobocupServerAddress;
import sk.fiit.testframework.trainer.Trainer;
import sk.fiit.testframework.worldrepresentation.ISimulationStateObserver;
import sk.fiit.testframework.worldrepresentation.models.SimulationState;

/**
 * Provides an abstract TestCase class. 
 * @author relation
 */
@UnderConstruction
public abstract class TestCase implements Callable<TestCaseResult>, ISimulationStateObserver {

    protected RobocupServerAddress address;
    protected Trainer trainer;

    protected Monitor monitor;
    protected SimulationState simulationState;
    protected boolean updated = false;
    private ExecutorService executorService = Executors.newCachedThreadPool();

    protected double initTime = 0;
    private boolean initialized = false;

    public void setInitTime(double initTime) {
        this.initTime = initTime;
    }

    public double getInitTime() {
        return initTime;
    }

    public double getElapsedTime() {
        return (monitor.getSimulationState().getGameStateInfo().getTime() - getInitTime());
    }

    public TestCase(RobocupServerAddress address) throws IOException {
        this.address = address;
        this.monitor = new Monitor(address.getServerMonitorAddress().getHostName(),
                address.getServerMonitorAddress().getPort());
        this.trainer = new Trainer(address, monitor.socket);

        monitor.getSimulationState().registerObserver(this);

        executorService.submit(new Runnable() {
            @Override
            public void run() {
                try {
                    monitor.start();
                } catch (IOException ex) {
                    Logger.getLogger(TestCase.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        });

    }

    public abstract void init();
    public abstract TestCaseResult evaluate(SimulationState ss);
    public abstract boolean isStopCriterionMet(SimulationState ss);

    @Override
    public TestCaseResult call() throws Exception {
        init();
        while (true) {
            if (isUpdated()) {
//                System.out.println("is updated = true");
                setUpdated(false);
                Thread.yield();
                if (!initialized)
                    continue;

                if (isStopCriterionMet(simulationState)) {
                    return evaluate(simulationState);
                }
            }
        }
    }

    @Override
    public synchronized void update() {
       // System.out.println("update()");
        simulationState = monitor.getSimulationState();
        if (!initialized) {
            if (monitor.isInformationComplete()) {
                setInitTime(simulationState.getGameStateInfo().getTime());
                initialized = true;
            }
        }
        setUpdated(true);
        notifyAll();
    }

    public synchronized boolean isUpdated() {
        return updated;
    }

    public synchronized void setUpdated(boolean updated) {
        this.updated = updated;
    }

}
